@@ -132,6 +132,8 @@ EGG_DEFINE_COUNTER (instances, "PhotosBaseItem", "Instances", "Number of PhotosB
 
 
 typedef struct _PhotosBaseItemSaveData PhotosBaseItemSaveData;
+typedef struct _PhotosBaseItemSaveBufferData PhotosBaseItemSaveBufferData;
+typedef struct _PhotosBaseItemSaveToStreamData PhotosBaseItemSaveToStreamData;
 
 struct _PhotosBaseItemSaveData
 {
@@ -139,6 +141,21 @@ struct _PhotosBaseItemSaveData
   GFile *unique_file;
   GeglBuffer *buffer;
   gchar *type;
+  gdouble zoom;
+};
+
+struct _PhotosBaseItemSaveBufferData
+{
+  GFile *file;
+  GFileOutputStream *stream;
+};
+
+struct _PhotosBaseItemSaveToStreamData
+{
+  GFile *file;
+  GFileIOStream *iostream;
+  GOutputStream *ostream;
+  gdouble zoom;
 };
 
 static GdkPixbuf *failed_icon;
@@ -151,7 +168,7 @@ static void photos_base_item_populate_from_cursor (PhotosBaseItem *self, Tracker
 
 
 static PhotosBaseItemSaveData *
-photos_base_item_save_data_new (GFile *dir, GeglBuffer *buffer, const gchar *type)
+photos_base_item_save_data_new (GFile *dir, GeglBuffer *buffer, const gchar *type, gdouble zoom)
 {
   PhotosBaseItemSaveData *data;
 
@@ -164,6 +181,7 @@ photos_base_item_save_data_new (GFile *dir, GeglBuffer *buffer, const gchar *typ
     data->buffer = g_object_ref (buffer);
 
   data->type = g_strdup (type);
+  data->zoom = zoom;
 
   return data;
 }
@@ -180,6 +198,51 @@ photos_base_item_save_data_free (PhotosBaseItemSaveData *data)
 }
 
 
+static PhotosBaseItemSaveBufferData *
+photos_base_item_save_buffer_data_new (GFile *file, GFileOutputStream *stream)
+{
+  PhotosBaseItemSaveBufferData *data;
+
+  data = g_slice_new0 (PhotosBaseItemSaveBufferData);
+  data->file = g_object_ref (file);
+  data->stream = g_object_ref (stream);
+
+  return data;
+}
+
+
+static void
+photos_base_item_save_buffer_data_free (PhotosBaseItemSaveBufferData *data)
+{
+  g_clear_object (&data->file);
+  g_clear_object (&data->stream);
+  g_slice_free (PhotosBaseItemSaveBufferData, data);
+}
+
+
+static PhotosBaseItemSaveToStreamData *
+photos_base_item_save_to_stream_data_new (GOutputStream *ostream, gdouble zoom)
+{
+  PhotosBaseItemSaveToStreamData *data;
+
+  data = g_slice_new0 (PhotosBaseItemSaveToStreamData);
+  data->ostream = g_object_ref (ostream);
+  data->zoom = zoom;
+
+  return data;
+}
+
+
+static void
+photos_base_item_save_to_stream_data_free (PhotosBaseItemSaveToStreamData *data)
+{
+  g_clear_object (&data->file);
+  g_clear_object (&data->iostream);
+  g_clear_object (&data->ostream);
+  g_slice_free (PhotosBaseItemSaveToStreamData, data);
+}
+
+
 static GdkPixbuf *
 photos_base_item_create_placeholder_icon (const gchar *icon_name)
 {
@@ -440,19 +503,13 @@ photos_base_item_default_open (PhotosBaseItem *self, GdkScreen *screen, guint32
 
   if (priv->default_app != NULL)
     {
-      GList *uris = NULL;
-
-      uris = g_list_prepend (uris, g_strdup (priv->uri));
-
       error = NULL;
-      g_app_info_launch_uris (priv->default_app, uris, NULL, &error);
+      photos_utils_app_info_launch_uri (priv->default_app, priv->uri, NULL, &error);
       if (error != NULL)
         {
           g_warning ("Unable to show URI %s: %s", priv->uri, error->message);
           g_error_free (error);
         }
-
-      g_list_free_full (uris, g_free);
     }
   else
     {
@@ -908,6 +965,63 @@ photos_base_item_guess_save_sizes_in_thread_func (GTask *task,
 }
 
 
+static void
+photos_base_item_guess_save_sizes_load (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
+  GError *error;
+  GTask *task = G_TASK (user_data);
+  GeglBuffer *buffer = NULL;
+  GeglNode *graph = NULL;
+  PhotosBaseItemSaveData *data;
+
+  error = NULL;
+  graph = photos_base_item_load_finish (self, res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  buffer = photos_utils_create_buffer_from_node (graph);
+  data = photos_base_item_save_data_new (NULL, buffer, self->priv->mime_type, 0.0);
+  g_task_set_task_data (task, data, (GDestroyNotify) photos_base_item_save_data_free);
+
+  g_task_run_in_thread (task, photos_base_item_guess_save_sizes_in_thread_func);
+
+ out:
+  g_clear_object (&buffer);
+  g_clear_object (&graph);
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_pipeline_is_edited_load (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
+  GError *error;
+  GTask *task = G_TASK (user_data);
+  GeglNode *graph = NULL;
+  gboolean is_edited;
+
+  error = NULL;
+  graph = photos_base_item_load_finish (self, res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  is_edited = photos_pipeline_is_edited (self->priv->pipeline);
+  g_task_return_boolean (task, is_edited);
+
+ out:
+  g_clear_object (&graph);
+  g_object_unref (task);
+}
+
+
 static gboolean
 photos_base_item_process_idle (gpointer user_data)
 {
@@ -1287,7 +1401,162 @@ photos_base_item_save_metadata_finish (PhotosBaseItem *self, GAsyncResult *res,
 
 
 static void
-photos_base_item_save_save_metadata (GObject *source_object, GAsyncResult *res, gpointer user_data)
+photos_base_item_save_buffer_save_metadata (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
+  GError *error;
+  GTask *task = G_TASK (user_data);
+
+  error = NULL;
+  if (!photos_base_item_save_metadata_finish (self, res, &error))
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_task_return_boolean (task, TRUE);
+
+ out:
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_save_buffer_stream_close (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItem *self;
+  PhotosBaseItemSaveBufferData *data;
+  GCancellable *cancellable;
+  GOutputStream *stream = G_OUTPUT_STREAM (source_object);
+  GError *error = NULL;
+
+  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
+  cancellable = g_task_get_cancellable (task);
+  data = (PhotosBaseItemSaveBufferData *) g_task_get_task_data (task);
+
+  if (!g_output_stream_close_finish (stream, res, &error))
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  photos_base_item_save_metadata_async (self,
+                                        data->file,
+                                        cancellable,
+                                        photos_base_item_save_buffer_save_metadata,
+                                        g_object_ref (task));
+
+ out:
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_save_buffer_save_to_stream (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItemSaveBufferData *data;
+  GCancellable *cancellable;
+  GError *error = NULL;
+
+  cancellable = g_task_get_cancellable (task);
+  data = (PhotosBaseItemSaveBufferData *) g_task_get_task_data (task);
+
+  if (!gdk_pixbuf_save_to_stream_finish (res, &error))
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_output_stream_close_async (G_OUTPUT_STREAM (data->stream),
+                               G_PRIORITY_DEFAULT,
+                               cancellable,
+                               photos_base_item_save_buffer_stream_close,
+                               g_object_ref (task));
+
+ out:
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_save_buffer_async (PhotosBaseItem *self,
+                                    GeglBuffer *buffer,
+                                    GFile *file,
+                                    GFileOutputStream *stream,
+                                    GCancellable *cancellable,
+                                    GAsyncReadyCallback callback,
+                                    gpointer user_data)
+{
+  PhotosBaseItemPrivate *priv = self->priv;
+  GTask *task = NULL;
+  GdkPixbuf *pixbuf = NULL;
+  GeglNode *buffer_source;
+  GeglNode *graph = NULL;
+  PhotosBaseItemSaveBufferData *data;
+
+  data = photos_base_item_save_buffer_data_new (file, stream);
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_base_item_save_buffer_async);
+  g_task_set_task_data (task, data, (GDestroyNotify) photos_base_item_save_buffer_data_free);
+
+  graph = gegl_node_new ();
+  buffer_source = gegl_node_new_child (graph, "operation", "gegl:buffer-source", "buffer", buffer, NULL);
+  pixbuf = photos_utils_create_pixbuf_from_node (buffer_source);
+  if (pixbuf == NULL)
+    {
+      g_task_return_new_error (task, PHOTOS_ERROR, 0, "Failed to create a GdkPixbuf from the GeglBuffer");
+      goto out;
+    }
+
+  if (g_strcmp0 (priv->mime_type, "image/png") == 0)
+    {
+      gdk_pixbuf_save_to_stream_async (pixbuf,
+                                       G_OUTPUT_STREAM (stream),
+                                       "png",
+                                       cancellable,
+                                       photos_base_item_save_buffer_save_to_stream,
+                                       g_object_ref (task),
+                                       NULL);
+    }
+  else
+    {
+      gdk_pixbuf_save_to_stream_async (pixbuf,
+                                       G_OUTPUT_STREAM (stream),
+                                       "jpeg",
+                                       cancellable,
+                                       photos_base_item_save_buffer_save_to_stream,
+                                       g_object_ref (task),
+                                       "quality", "90",
+                                       NULL);
+    }
+
+ out:
+  g_clear_object (&graph);
+  g_clear_object (&pixbuf);
+  g_clear_object (&task);
+}
+
+
+static gboolean
+photos_base_item_save_buffer_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
+{
+  GTask *task;
+
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
+  task = G_TASK (res);
+
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_save_buffer_async, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  return g_task_propagate_boolean (task, error);
+}
+
+
+static void
+photos_base_item_save_save_buffer (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
   PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
   GError *error;
@@ -1297,7 +1566,7 @@ photos_base_item_save_save_metadata (GObject *source_object, GAsyncResult *res,
   data = (PhotosBaseItemSaveData *) g_task_get_task_data (task);
 
   error = NULL;
-  if (!photos_base_item_save_metadata_finish (self, res, &error))
+  if (!photos_base_item_save_buffer_finish (self, res, &error))
     {
       g_task_return_error (task, error);
       goto out;
@@ -1311,49 +1580,18 @@ photos_base_item_save_save_metadata (GObject *source_object, GAsyncResult *res,
 
 
 static void
-photos_base_item_save_save_to_stream (GObject *source_object, GAsyncResult *res, gpointer user_data)
+photos_base_item_save_file_create (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
-  GTask *task = G_TASK (user_data);
   PhotosBaseItem *self;
-  PhotosBaseItemSaveData *data;
-  GCancellable *cancellable;
-  GError *error = NULL;
-
-  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
-  cancellable = g_task_get_cancellable (task);
-  data = (PhotosBaseItemSaveData *) g_task_get_task_data (task);
-
-  if (!gdk_pixbuf_save_to_stream_finish (res, &error))
-    {
-      g_task_return_error (task, error);
-      goto out;
-    }
-
-  photos_base_item_save_metadata_async (self,
-                                        data->unique_file,
-                                        cancellable,
-                                        photos_base_item_save_save_metadata,
-                                        g_object_ref (task));
-
- out:
-  g_object_unref (task);
-}
-
-
-static void
-photos_base_item_save_create (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
   GTask *task = G_TASK (user_data);
   GCancellable *cancellable;
-  GdkPixbuf *pixbuf = NULL;
   GError *error = NULL;
-  GeglNode *buffer_source;
-  GeglNode *graph = NULL;
   GFile *file = G_FILE (source_object);
   GFile *unique_file = NULL;
   GFileOutputStream *stream = NULL;
   PhotosBaseItemSaveData *data;
 
+  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
   cancellable = g_task_get_cancellable (task);
   data = (PhotosBaseItemSaveData *) g_task_get_task_data (task);
 
@@ -1368,40 +1606,15 @@ photos_base_item_save_create (GObject *source_object, GAsyncResult *res, gpointe
   g_assert_true (G_IS_FILE (unique_file));
   data->unique_file = g_object_ref (unique_file);
 
-  graph = gegl_node_new ();
-  buffer_source = gegl_node_new_child (graph, "operation", "gegl:buffer-source", "buffer", data->buffer, NULL);
-  pixbuf = photos_utils_create_pixbuf_from_node (buffer_source);
-  if (pixbuf == NULL)
-    {
-      g_task_return_new_error (task, PHOTOS_ERROR, 0, "Failed to create a GdkPixbuf from the GeglBuffer");
-      goto out;
-    }
-
-  if (g_strcmp0 (data->type, "jpeg") == 0)
-    {
-      gdk_pixbuf_save_to_stream_async (pixbuf,
-                                       G_OUTPUT_STREAM (stream),
-                                       data->type,
-                                       cancellable,
-                                       photos_base_item_save_save_to_stream,
-                                       g_object_ref (task),
-                                       "quality", "90",
-                                       NULL);
-    }
-  else
-    {
-      gdk_pixbuf_save_to_stream_async (pixbuf,
-                                       G_OUTPUT_STREAM (stream),
-                                       data->type,
-                                       cancellable,
-                                       photos_base_item_save_save_to_stream,
-                                       g_object_ref (task),
-                                       NULL);
-    }
+  photos_base_item_save_buffer_async (self,
+                                      data->buffer,
+                                      unique_file,
+                                      stream,
+                                      cancellable,
+                                      photos_base_item_save_save_buffer,
+                                      g_object_ref (task));
 
  out:
-  g_clear_object (&graph);
-  g_clear_object (&pixbuf);
   g_clear_object (&stream);
   g_clear_object (&unique_file);
   g_object_unref (task);
@@ -1440,7 +1653,7 @@ photos_base_item_save_buffer_zoom (GObject *source_object, GAsyncResult *res, gp
                                   G_FILE_CREATE_NONE,
                                   G_PRIORITY_DEFAULT,
                                   cancellable,
-                                  photos_base_item_save_create,
+                                  photos_base_item_save_file_create,
                                   g_object_ref (task));
 
  out:
@@ -1450,6 +1663,233 @@ photos_base_item_save_buffer_zoom (GObject *source_object, GAsyncResult *res, gp
 }
 
 
+static void
+photos_base_item_save_to_stream_file_delete (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GError *error;
+  GFile *file = G_FILE (source_object);
+  GTask *task = G_TASK (user_data);
+
+  error = NULL;
+  if (!g_file_delete_finish (file, res, &error))
+    {
+      gchar *uri = NULL;
+
+      uri = g_file_get_uri (file);
+      g_warning ("Unable to delete temporary file %s: %s", uri, error->message);
+      g_free (uri);
+      g_error_free (error);
+    }
+
+  /* Mark the task as a success, no matter what. The item has already
+   * been saved to the GOutputStream, so it doesn't make sense to
+   * fail the task just because we failed to delete the temporary
+   * file created during the process.
+   */
+  g_task_return_boolean (task, TRUE);
+
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_save_to_stream_stream_close (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GCancellable *cancellable;
+  GError *error;
+  GIOStream *iostream = G_IO_STREAM (source_object);
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItemSaveToStreamData *data;
+
+  cancellable = g_task_get_cancellable (task);
+  data = (PhotosBaseItemSaveToStreamData *) g_task_get_task_data (task);
+
+  error = NULL;
+  g_io_stream_close_finish (iostream, res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_file_delete_async (data->file,
+                       G_PRIORITY_DEFAULT,
+                       cancellable,
+                       photos_base_item_save_to_stream_file_delete,
+                       g_object_ref (task));
+
+ out:
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_save_to_stream_stream_splice (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GCancellable *cancellable;
+  GError *error;
+  GOutputStream *ostream = G_OUTPUT_STREAM (source_object);
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItemSaveToStreamData *data;
+
+  cancellable = g_task_get_cancellable (task);
+  data = (PhotosBaseItemSaveToStreamData *) g_task_get_task_data (task);
+
+  error = NULL;
+  g_output_stream_splice_finish (ostream, res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_io_stream_close_async (G_IO_STREAM (data->iostream),
+                           G_PRIORITY_DEFAULT,
+                           cancellable,
+                           photos_base_item_save_to_stream_stream_close,
+                           g_object_ref (task));
+
+ out:
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_save_to_stream_save_buffer (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
+  GCancellable *cancellable;
+  GError *error;
+  GInputStream *istream;
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItemSaveToStreamData *data;
+
+  cancellable = g_task_get_cancellable (task);
+  data = (PhotosBaseItemSaveToStreamData *) g_task_get_task_data (task);
+
+  error = NULL;
+  if (!photos_base_item_save_buffer_finish (self, res, &error))
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  istream = g_io_stream_get_input_stream (G_IO_STREAM (data->iostream));
+  g_assert_true (g_seekable_can_seek (G_SEEKABLE (istream)));
+
+  error = NULL;
+  if (!g_seekable_seek (G_SEEKABLE (istream), 0, G_SEEK_SET, cancellable, &error))
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_output_stream_splice_async (data->ostream,
+                                istream,
+                                G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+                                G_PRIORITY_DEFAULT,
+                                cancellable,
+                                photos_base_item_save_to_stream_stream_splice,
+                                g_object_ref (task));
+
+ out:
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_save_to_stream_buffer_zoom (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItem *self;
+  GCancellable *cancellable;
+  GError *error;
+  GFile *file = NULL;
+  GFileIOStream *iostream = NULL;
+  GOutputStream *ostream;
+  GeglBuffer *buffer = GEGL_BUFFER (source_object);
+  GeglBuffer *buffer_zoomed = NULL;
+  PhotosBaseItemSaveToStreamData *data;
+
+  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
+  cancellable = g_task_get_cancellable (task);
+  data = (PhotosBaseItemSaveToStreamData *) g_task_get_task_data (task);
+
+  error = NULL;
+  buffer_zoomed = photos_utils_buffer_zoom_finish (buffer, res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  error = NULL;
+  file = g_file_new_tmp (PACKAGE_TARNAME "-XXXXXX", &iostream, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_assert_null (data->file);
+  data->file = g_object_ref (file);
+
+  g_assert_null (data->iostream);
+  data->iostream = g_object_ref (iostream);
+
+  ostream = g_io_stream_get_output_stream (G_IO_STREAM (iostream));
+  photos_base_item_save_buffer_async (self,
+                                      buffer_zoomed,
+                                      file,
+                                      G_FILE_OUTPUT_STREAM (ostream),
+                                      cancellable,
+                                      photos_base_item_save_to_stream_save_buffer,
+                                      g_object_ref (task));
+
+ out:
+  g_clear_object (&buffer_zoomed);
+  g_clear_object (&file);
+  g_clear_object (&iostream);
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_save_to_stream_load (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
+  GCancellable *cancellable;
+  GError *error;
+  GTask *task = G_TASK (user_data);
+  GeglBuffer *buffer = NULL;
+  GeglNode *graph = NULL;
+  PhotosBaseItemSaveToStreamData *data;
+
+  cancellable = g_task_get_cancellable (task);
+  data = (PhotosBaseItemSaveToStreamData *) g_task_get_task_data (task);
+
+  error = NULL;
+  graph = photos_base_item_load_finish (self, res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  buffer = photos_utils_create_buffer_from_node (graph);
+  photos_utils_buffer_zoom_async (buffer,
+                                  data->zoom,
+                                  cancellable,
+                                  photos_base_item_save_to_stream_buffer_zoom,
+                                  g_object_ref (task));
+
+ out:
+  g_clear_object (&buffer);
+  g_clear_object (&graph);
+  g_object_unref (task);
+}
+
+
 static void
 photos_base_item_set_thumbnailing_icon (PhotosBaseItem *self)
 {
@@ -2302,34 +2742,16 @@ photos_base_item_guess_save_sizes_async (PhotosBaseItem *self,
                                          GAsyncReadyCallback callback,
                                          gpointer user_data)
 {
-  PhotosBaseItemPrivate *priv;
-  GeglBuffer *buffer;
-  GeglNode *graph;
   GTask *task;
-  PhotosBaseItemSaveData *data;
 
   g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
-  priv = self->priv;
-
-  g_return_if_fail (!priv->collection);
-  g_return_if_fail (priv->edit_graph != NULL);
-  g_return_if_fail (priv->load_graph != NULL);
-  g_return_if_fail (priv->pipeline != NULL);
-  g_return_if_fail (priv->processor != NULL);
-  g_return_if_fail (!gegl_processor_work (priv->processor, NULL));
-
-  graph = photos_pipeline_get_graph (priv->pipeline);
-  buffer = photos_utils_create_buffer_from_node (graph);
-
-  data = photos_base_item_save_data_new (NULL, buffer, priv->mime_type);
+  g_return_if_fail (!self->priv->collection);
 
   task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (task, photos_base_item_guess_save_sizes_async);
-  g_task_set_task_data (task, data, (GDestroyNotify) photos_base_item_save_data_free);
 
-  g_task_run_in_thread (task, photos_base_item_guess_save_sizes_in_thread_func);
+  photos_base_item_load_async (self, cancellable, photos_base_item_guess_save_sizes_load, g_object_ref (task));
 
-  g_object_unref (buffer);
   g_object_unref (task);
 }
 
@@ -2643,6 +3065,83 @@ photos_base_item_operations_revert_finish (PhotosBaseItem *self, GAsyncResult *r
 }
 
 
+void
+photos_base_item_pipeline_is_edited_async (PhotosBaseItem *self,
+                                           GCancellable *cancellable,
+                                           GAsyncReadyCallback callback,
+                                           gpointer user_data)
+{
+  PhotosBaseItemPrivate *priv;
+  GTask *task;
+
+  g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
+  priv = self->priv;
+
+  g_return_if_fail (!priv->collection);
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_base_item_pipeline_is_edited_async);
+
+  photos_base_item_load_async (self, cancellable, photos_base_item_pipeline_is_edited_load, g_object_ref (task));
+
+  g_object_unref (task);
+}
+
+
+gboolean
+photos_base_item_pipeline_is_edited_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
+{
+  GTask *task = G_TASK (res);
+
+  g_return_val_if_fail (PHOTOS_IS_BASE_ITEM (self), FALSE);
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_pipeline_is_edited_async, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  return g_task_propagate_boolean (task, error);
+}
+
+
+void
+photos_base_item_pipeline_revert_to_original_async (PhotosBaseItem *self,
+                                                    GCancellable *cancellable,
+                                                    GAsyncReadyCallback callback,
+                                                    gpointer user_data)
+
+{
+  PhotosBaseItemPrivate *priv;
+  GTask *task;
+
+  g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
+  priv = self->priv;
+
+  g_return_if_fail (!priv->collection);
+
+  photos_pipeline_revert_to_original (priv->pipeline);
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_base_item_pipeline_revert_to_original_async);
+
+  photos_base_item_process_async (self, cancellable, photos_base_item_common_process, g_object_ref (task));
+
+  g_object_unref (task);
+}
+
+
+gboolean
+photos_base_item_pipeline_revert_to_original_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
+{
+  GTask *task = G_TASK (res);
+
+  g_return_val_if_fail (PHOTOS_IS_BASE_ITEM (self), FALSE);
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_pipeline_revert_to_original_async, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  return g_task_propagate_boolean (task, error);
+}
+
+
 void
 photos_base_item_pipeline_save_async (PhotosBaseItem *self,
                                       GCancellable *cancellable,
@@ -2733,6 +3232,42 @@ photos_base_item_refresh (PhotosBaseItem *self)
 }
 
 
+static void
+photos_base_item_save_load (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
+  GCancellable *cancellable;
+  GError *error;
+  GTask *task = G_TASK (user_data);
+  GeglBuffer *buffer = NULL;
+  GeglNode *graph = NULL;
+  PhotosBaseItemSaveData *data;
+
+  cancellable = g_task_get_cancellable (task);
+  data = (PhotosBaseItemSaveData *) g_task_get_task_data (task);
+
+  error = NULL;
+  graph = photos_base_item_load_finish (self, res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  buffer = photos_utils_create_buffer_from_node (graph);
+  photos_utils_buffer_zoom_async (buffer,
+                                  data->zoom,
+                                  cancellable,
+                                  photos_base_item_save_buffer_zoom,
+                                  g_object_ref (task));
+
+ out:
+  g_clear_object (&buffer);
+  g_clear_object (&graph);
+  g_object_unref (task);
+}
+
+
 void
 photos_base_item_save_async (PhotosBaseItem *self,
                              GFile *dir,
@@ -2743,43 +3278,23 @@ photos_base_item_save_async (PhotosBaseItem *self,
 {
   PhotosBaseItemPrivate *priv;
   GTask *task;
-  GeglBuffer *buffer;
-  GeglNode *graph;
   PhotosBaseItemSaveData *data;
-  const gchar *type;
 
   g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
   priv = self->priv;
 
   g_return_if_fail (!priv->collection);
   g_return_if_fail (G_IS_FILE (dir));
-  g_return_if_fail (priv->edit_graph != NULL);
   g_return_if_fail (priv->filename != NULL && priv->filename[0] != '\0');
-  g_return_if_fail (priv->load_graph != NULL);
-  g_return_if_fail (priv->pipeline != NULL);
-  g_return_if_fail (priv->processor != NULL);
-  g_return_if_fail (!gegl_processor_work (priv->processor, NULL));
 
-  if (g_strcmp0 (priv->mime_type, "image/png") == 0)
-    type = "png";
-  else
-    type = "jpeg";
-
-  data = photos_base_item_save_data_new (dir, NULL, type);
+  data = photos_base_item_save_data_new (dir, NULL, NULL, zoom);
 
   task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (task, photos_base_item_save_async);
   g_task_set_task_data (task, data, (GDestroyNotify) photos_base_item_save_data_free);
 
-  graph = photos_pipeline_get_graph (priv->pipeline);
-  buffer = photos_utils_create_buffer_from_node (graph);
-  photos_utils_buffer_zoom_async (buffer,
-                                  zoom,
-                                  cancellable,
-                                  photos_base_item_save_buffer_zoom,
-                                  g_object_ref (task));
+  photos_base_item_load_async (self, cancellable, photos_base_item_save_load, g_object_ref (task));
 
-  g_object_unref (buffer);
   g_object_unref (task);
 }
 
@@ -2798,6 +3313,54 @@ photos_base_item_save_finish (PhotosBaseItem *self, GAsyncResult *res, GError **
 }
 
 
+void
+photos_base_item_save_to_stream_async (PhotosBaseItem *self,
+                                       GOutputStream *stream,
+                                       gdouble zoom,
+                                       GCancellable *cancellable,
+                                       GAsyncReadyCallback callback,
+                                       gpointer user_data)
+{
+  PhotosBaseItemPrivate *priv;
+  GTask *task;
+  PhotosBaseItemSaveToStreamData *data;
+
+  g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
+  priv = self->priv;
+
+  g_return_if_fail (!priv->collection);
+  g_return_if_fail (G_IS_OUTPUT_STREAM (stream));
+  g_return_if_fail (priv->filename != NULL && priv->filename[0] != '\0');
+
+  data = photos_base_item_save_to_stream_data_new (stream, zoom);
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_base_item_save_to_stream_async);
+  g_task_set_task_data (task, data, (GDestroyNotify) photos_base_item_save_to_stream_data_free);
+
+  photos_base_item_load_async (self, cancellable, photos_base_item_save_to_stream_load, g_object_ref (task));
+
+  g_object_unref (task);
+}
+
+
+gboolean
+photos_base_item_save_to_stream_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
+{
+  GTask *task;
+
+  g_return_val_if_fail (PHOTOS_IS_BASE_ITEM (self), FALSE);
+
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
+  task = G_TASK (res);
+
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_save_to_stream_async, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  return g_task_propagate_boolean (task, error);
+}
+
+
 void
 photos_base_item_set_default_app (PhotosBaseItem *self, GAppInfo *default_app)
 {
