@@ -66,7 +66,10 @@ struct _PhotosBaseManagerClass
 
   /* virtual methods */
   void (*add_object)      (PhotosBaseManager *self, GObject *object);
+  GObject *(*get_active_object) (PhotosBaseManager *self);
   gchar *(*get_filter)    (PhotosBaseManager *self, gint flags);
+  GObject *(*get_object_by_id) (PhotosBaseManager *self, const gchar *id);
+  GHashTable *(*get_objects) (PhotosBaseManager *self);
   gchar *(*get_where)    (PhotosBaseManager *self, gint flags);
   void (*remove_object_by_id) (PhotosBaseManager *self, const gchar *id);
   gboolean (*set_active_object)    (PhotosBaseManager *self, GObject *object);
@@ -80,6 +83,8 @@ struct _PhotosBaseManagerClass
 
 GType               photos_base_manager_get_type                 (void) G_GNUC_CONST;
 
+PhotosBaseManager  *photos_base_manager_new                      (void);
+
 void                photos_base_manager_add_object               (PhotosBaseManager *self, GObject *object);
 
 void                photos_base_manager_clear                    (PhotosBaseManager *self);
