@@ -85,8 +85,12 @@ PhotosBaseManager        *photos_item_manager_new                (void);
 
 void                      photos_item_manager_activate_previous_collection (PhotosItemManager *self);
 
-void                      photos_item_manager_add_item           (PhotosItemManager *self,
-                                                                  TrackerSparqlCursor *cursor);
+void                      photos_item_manager_add_item              (PhotosItemManager *self,
+                                                                     TrackerSparqlCursor *cursor);
+
+void                      photos_item_manager_add_item_for_mode     (PhotosItemManager *self,
+                                                                     PhotosWindowMode mode,
+                                                                     TrackerSparqlCursor *cursor);
 
 PhotosBaseItem           *photos_item_manager_create_item        (PhotosItemManager *self,
                                                                   TrackerSparqlCursor *cursor);
@@ -97,6 +101,10 @@ GHashTable               *photos_item_manager_get_collections       (PhotosItemM
 
 PhotosLoadState           photos_item_manager_get_load_state        (PhotosItemManager *self);
 
+void                      photos_item_manager_hide_item             (PhotosItemManager *self, PhotosBaseItem *item);
+
+void                      photos_item_manager_unhide_item           (PhotosItemManager *self, PhotosBaseItem *item);
+
 gboolean                  photos_mode_controller_get_can_fullscreen (PhotosModeController *self);
 
 gboolean                  photos_mode_controller_get_fullscreen     (PhotosModeController *self);
