@@ -94,19 +94,19 @@ static void
 photos_search_type_manager_init (PhotosSearchTypeManager *self)
 {
   PhotosSearchType *search_type;
-  const gchar *item_filter;
+  gchar *item_filter;
   gchar *all_filter;
   gchar *col_filter;
-  gchar *fav_filter;
+  gchar *gif_filter;
 
-  item_filter = "fn:contains (?type, 'nmm#Photo')";
+  gif_filter = g_strdup ("nie:mimeType(?urn) != 'image/gif'");
+  item_filter = g_strdup_printf ("(fn:contains (?type, 'nmm#Photo') && %s)", gif_filter);
   col_filter = g_strdup_printf ("(fn:contains (?type, 'nfo#DataContainer')"
                                 " && ?count > 0"
                                 " && (fn:starts-with (nao:identifier (?urn), '%s')"
                                 "     || (?urn = nfo:image-category-screenshot)))",
                                 PHOTOS_QUERY_COLLECTIONS_IDENTIFIER);
   all_filter = g_strdup_printf ("(%s || %s)", col_filter, item_filter);
-  fav_filter = g_strdup_printf ("(%s || %s)", col_filter, item_filter);
 
   search_type = photos_search_type_new_full (PHOTOS_SEARCH_TYPE_STOCK_ALL,
                                              _("All"),
@@ -126,24 +126,24 @@ photos_search_type_manager_init (PhotosSearchTypeManager *self)
 
   search_type = photos_search_type_new_full (PHOTOS_SEARCH_TYPE_STOCK_FAVORITES,
                                              _("Favorites"),
-                                             "?urn a rdfs:Resource; nao:hasTag nao:predefined-tag-favorite. "
-                                             "OPTIONAL {?item a nie:InformationElement; nie:isPartOf ?urn}",
-                                             fav_filter);
+                                             "?urn a nmm:Photo; nao:hasTag nao:predefined-tag-favorite. ",
+                                             gif_filter);
   photos_base_manager_add_object (PHOTOS_BASE_MANAGER (self), G_OBJECT (search_type));
   g_object_unref (search_type);
 
   search_type = photos_search_type_new_full (PHOTOS_SEARCH_TYPE_STOCK_PHOTOS,
                                              _("Photos"),
                                              "?urn a nmm:Photo",
-                                             "(true)");
+                                             gif_filter);
   photos_base_manager_add_object (PHOTOS_BASE_MANAGER (self), G_OBJECT (search_type));
   g_object_unref (search_type);
 
   photos_base_manager_set_active_object_by_id (PHOTOS_BASE_MANAGER (self), PHOTOS_SEARCH_TYPE_STOCK_PHOTOS);
 
+  g_free (item_filter);
   g_free (all_filter);
   g_free (col_filter);
-  g_free (fav_filter);
+  g_free (gif_filter);
 }
 
 
