@@ -51,6 +51,10 @@
 #include "photos-operation-png-guess-sizes.h"
 #include "photos-operation-saturation.h"
 #include "photos-query.h"
+#include "photos-share-point.h"
+#include "photos-share-point-email.h"
+#include "photos-share-point-google.h"
+#include "photos-share-point-online.h"
 #include "photos-source.h"
 #include "photos-tool.h"
 #include "photos-tool-colors.h"
@@ -81,6 +85,24 @@ struct _PhotosUtilsFileCreateData
 };
 
 
+gboolean
+photos_utils_app_info_launch_uri (GAppInfo *appinfo, const gchar *uri, GAppLaunchContext *launch_context, GError **error)
+{
+  GList *uris = NULL;
+  gboolean ret_val;
+
+  g_return_val_if_fail (G_IS_APP_INFO (appinfo), FALSE);
+  g_return_val_if_fail (uri != NULL && uri[0] != '\0', FALSE);
+  g_return_val_if_fail (launch_context == NULL || G_IS_APP_LAUNCH_CONTEXT (launch_context), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  uris = g_list_prepend (uris, g_strdup (uri));
+  ret_val = g_app_info_launch_uris (appinfo, uris, launch_context, error);
+  g_list_free_full (uris, g_free);
+  return ret_val;
+}
+
+
 static void
 photos_utils_put_pixel (guchar *p)
 {
@@ -829,6 +851,9 @@ photos_utils_ensure_builtins (void)
       g_type_ensure (PHOTOS_TYPE_OPERATION_PNG_GUESS_SIZES);
       g_type_ensure (PHOTOS_TYPE_OPERATION_SATURATION);
 
+      g_type_ensure (PHOTOS_TYPE_SHARE_POINT_EMAIL);
+      g_type_ensure (PHOTOS_TYPE_SHARE_POINT_GOOGLE);
+
       g_type_ensure (PHOTOS_TYPE_TOOL_COLORS);
       g_type_ensure (PHOTOS_TYPE_TOOL_CROP);
       g_type_ensure (PHOTOS_TYPE_TOOL_ENHANCE);
@@ -856,6 +881,12 @@ photos_utils_ensure_extension_points (void)
       extension_point = g_io_extension_point_register (PHOTOS_BASE_ITEM_EXTENSION_POINT_NAME);
       g_io_extension_point_set_required_type (extension_point, PHOTOS_TYPE_BASE_ITEM);
 
+      extension_point = g_io_extension_point_register (PHOTOS_SHARE_POINT_EXTENSION_POINT_NAME);
+      g_io_extension_point_set_required_type (extension_point, PHOTOS_TYPE_SHARE_POINT);
+
+      extension_point = g_io_extension_point_register (PHOTOS_SHARE_POINT_ONLINE_EXTENSION_POINT_NAME);
+      g_io_extension_point_set_required_type (extension_point, PHOTOS_TYPE_SHARE_POINT_ONLINE);
+
       extension_point = g_io_extension_point_register (PHOTOS_TOOL_EXTENSION_POINT_NAME);
       g_io_extension_point_set_required_type (extension_point, PHOTOS_TYPE_TOOL);
 
