@@ -50,6 +50,8 @@ G_BEGIN_DECLS
 #define PHOTOS_ORIENTATION_TOP (photos_utils_orientation_top_quark ())
 
 #define PHOTOS_BASE_ITEM_EXTENSION_POINT_NAME "photos-base-item"
+#define PHOTOS_SHARE_POINT_EXTENSION_POINT_NAME "photos-share-point"
+#define PHOTOS_SHARE_POINT_ONLINE_EXTENSION_POINT_NAME "photos-share-point-online"
 #define PHOTOS_TOOL_EXTENSION_POINT_NAME "photos-tool"
 #define PHOTOS_TRACKER_CONTROLLER_EXTENSION_POINT_NAME "photos-tracker-controller"
 
@@ -57,6 +59,11 @@ G_BEGIN_DECLS
   "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#image-category-screenshot"
 #define PHOTOS_EXPORT_SUBPATH "Exports"
 
+gboolean         photos_utils_app_info_launch_uri         (GAppInfo *appinfo,
+                                                           const gchar *uri,
+                                                           GAppLaunchContext *launch_context,
+                                                           GError **error);
+
 void             photos_utils_border_pixbuf               (GdkPixbuf *pixbuf);
 
 void             photos_utils_buffer_zoom_async           (GeglBuffer *buffer,
